library(ggplot2)
library(dplyr)
library(tidyverse)
library(readr)
library(readxl)
library(DataExplorer)
library(ggdist)
library(gridExtra)
library(GGally)
library(rstanarm)
library(caret)
library(ggpubr)

# Set theme for ggplot
mytheme = theme_bw()
theme_set(mytheme)  

d <- read.csv("Data.csv", header = TRUE, sep = ",", dec = ".")

SEED <- 123

# Figure 1

d2 <- d  %>% 
  rename(height=Height, `external valgus torque`=Elbow_Adduction_MER,
         weight=Weight, `pelvis PAV`=Pelvis_PAV, `trunk PAV`=Trunk_PAV,
         `separation time`= Separation_Pelvis_Trunk, player=Participant) %>% 
  pivot_longer(cols=3:7, names_to="predictor", values_to="value") %>% 
  mutate(predictor=fct_relevel(predictor, "pelvis PAV", "trunk PAV", "separation time", "weight", "height"))

p <- d2 %>% ggplot(aes(x=value,y=`external valgus torque`, colour=player)) +
  geom_point(alpha=0.5) + geom_smooth(method="lm", se=F) + facet_wrap(~predictor, scales="free") + 
  theme(legend.position = c(.85, 0.23)) + labs(x="")

# standardise the data: this is important as we set priors to be centred at zero
preProcValues <- preProcess(d, method = c("center", "scale"))
dT <- predict(preProcValues, d) 

# Fitting Bayesian hierarchical models
# Predictors: Pelvis_PAV - pelvis peak angular velocity (P)
#             Trunk_PAV - trunk peak angular velocity (T)
#             Separation_Pelvis_Trunk - separation time (S)
#             Weight - weight (W)
#             Height - height (H)
#             Elbow_Adduction_MER - external valgus torque

# PTS WH
outPTSWH <-  stan_lmer(Elbow_Adduction_MER ~ 0 +  (1 | Participant) + (0 + Pelvis_PAV + Trunk_PAV + Separation_Pelvis_Trunk | Participant) + Weight + Height ,
                       data=dT, adapt_delta = 0.99, chains = 4, seed = SEED)

# PT WH
outPTWH <-  stan_lmer(Elbow_Adduction_MER ~ 0 +  (1 | Participant) + (0 + Pelvis_PAV + Trunk_PAV | Participant) + Weight + Height ,
                      data=dT,  chains = 4, seed = SEED)

# TS WH
outTSWH <-  stan_lmer(Elbow_Adduction_MER ~ 0 +  (1 | Participant) + (0 + Trunk_PAV + Separation_Pelvis_Trunk | Participant) + Weight + Height ,
                      data=dT,  chains = 4, seed = SEED)

# PS WH
outPSWH <-  stan_lmer(Elbow_Adduction_MER ~ 0 +  (1 | Participant) + (0 + Pelvis_PAV + Separation_Pelvis_Trunk| Participant) + Weight + Height ,
                      data=dT,  chains = 4, seed = SEED)

# P WH
outPWH <-  stan_lmer(Elbow_Adduction_MER ~ 0 +  (1 | Participant) + (0 + Pelvis_PAV | Participant) + Weight + Height ,
                     data=dT,  chains = 4, seed = SEED)

# T WH
outTWH <-  stan_lmer(Elbow_Adduction_MER ~ 0 +  (1 | Participant) + (0 + Trunk_PAV | Participant) + Weight + Height ,
                     data=dT,  chains = 4, seed = SEED)

# S WH
outSWH <-  stan_lmer(Elbow_Adduction_MER ~ 0 +  (1 | Participant) + (0 + Separation_Pelvis_Trunk | Participant) + Weight + Height ,
                     data=dT,  chains = 4, seed = SEED)

# Leave-one-group-out cross-validation
set.seed(779)

cv <- loo::kfold_split_grouped(K = 11, x = dT$Participant)

lPTSWH_group <- rstanarm::kfold(outPTSWH, K=11, folds = cv, save_psis = TRUE) 
lPTWH_group <- rstanarm::kfold(outPTWH, K=11, folds = cv)
lTSWH_group<- rstanarm::kfold(outTSWH, K=11, folds = cv)
lPSWH_group <- rstanarm::kfold(outPSWH, K=11, folds = cv)
lPWH_group<- rstanarm::kfold(outPWH, K=11, folds = cv)
lTWH_group<- rstanarm::kfold(outTWH, K=11, folds = cv)
lSWH_group <- rstanarm::kfold(outSWH, K=11, folds = cv)

comparison_group <- loo_compare(lPTSWH_group, lPTWH_group, lTSWH_group, lPSWH_group, lPWH_group, lTWH_group, lSWH_group)

# Figure 2a
comparison <- data.frame(comparison_group) %>% mutate(model=substring(rownames(data.frame(comparison_group)),4))

logo <- ggplot(comparison,aes(x=fct_reorder(model, abs(elpd_diff)),y=abs(elpd_diff), color=model))+ 
  geom_errorbar(aes(ymin = abs(elpd_diff) - se_diff, ymax = abs(elpd_diff) + se_diff)) + 
  geom_point() + 
  ylab("elpd difference") +
  geom_text(aes(label = model), hjust = 1.1, vjust = 0.08) +
  ggtitle("a) elpd difference to model PTWH") +
  theme(legend.position = "none",
        axis.title.x = element_blank(),
        axis.ticks.x = element_blank(),
        axis.text.x = element_blank())

set.seed(878)

# Figure 2b
scat_PTWH <- pp_check(outPTWH, plotfun = "scatter_avg") + ggplot2::ggtitle("b) Posterior predictive check for model PTWH")

# Figure 2c
scat_TWH <- pp_check(outTWH, plotfun = "scatter_avg") + ggplot2::ggtitle("c) Posterior predictive check for model TWH")

# Figure 2
ggarrange(logo, scat_PTWH, scat_TWH, ncol = 1, nrow = 3, widths = 7, heights = 6)

# Appendix

# Figure 3
plot(outPTWH)